#!/usr/bin/env python3
# sndslib by @undersfx

r"""
Facilita a administração dos IPs listados no painel Sender Network Data Service (Microsoft).

Exemplo de Uso:

    >>> from sndslib import sndslib
    >>> r = sndslib.get_ip_status('mykey')
    >>> blocked_ips = sndslib.list_blocked_ips(r)
    [1.1.1.1, 2.2.2.2, 3.3.3.3]

    >>> r = sndslib.get_data('mykey')
    >>> sndslib.summarize(r)
    {'red': 272, 'green': 710, 'yellow': 852, 'traps': 1298, 'ips': 1834, 'date': '12/31/2019'}

    >>> sndslib.search_ip_status('3.3.3.3', r)
    {'activity_end': '12/31/2019 7:00 PM',
    'activity_start': '12/31/2019 10:00 AM',
    'comments': '',
    'complaint_rate': '< 0.1%',
    'data_commands': '1894',
    'filter_result': 'GREEN',
    'ip_address': '3.3.3.3',
    'message_recipients': '1894',
    'rcpt_commands': '1895',
    'sample_helo': '',
    'sample_mailfrom': '',
    'trap_message_end': '',
    'trap_message_start': '',
    'traphits': '0'}
"""

from urllib.request import urlopen
from datetime import datetime
import socket
import re


def get_ip_status(key):
    """Busca os ranges bloqueados no SNDS Automated Data Access."""

    response = urlopen(f'https://sendersupport.olc.protection.outlook.com/snds/ipStatus.aspx?key={key}')

    assert response.status == 200, 'Invalid return code: {}'.format(response.status)

    csv = list(response.read().decode('utf-8').split('\r\n'))

    csv = list(filter(None, csv))

    return csv


def get_data(key, date=None):
    """Busca os dados de uso dos IP no SNDS Automated Data Access."""

    if date:
        response = urlopen(f'https://sendersupport.olc.protection.outlook.com/snds/data.aspx?key={key}&date={date}')
    else:
        response = urlopen(f'https://sendersupport.olc.protection.outlook.com/snds/data.aspx?key={key}')

    assert response.status == 200, 'Invalid return code: {}'.format(response.status)

    csv = list(response.read().decode('utf-8').split('\r\n'))

    csv = list(filter(None, csv))

    return csv


def summarize(response):
    """Recebe a tabela com dados de uso dos IPs (sndslib.get_data) e retorna o status geral.

    >>> r = sndslib.get_data('mykey')
    >>> sndslib.summarize(r)
    {'red': 272, 'green': 710, 'yellow': 852, 'traps': 1298, 'ips': 1834, 'date': '12/31/2019'}
    """

    # Contagem de incidências do status e total de spamtraps
    summary = {'red': 0, 'green': 0, 'yellow': 0, 'traps': 0, 'ips': len(response), 'date': ''}

    for ip_status in response:
        status = format_ip_data(ip_status.split(','))

        if status['filter_result'] == 'GREEN':
            summary['green'] += 1
        elif status['filter_result'] == 'YELLOW':
            summary['yellow'] += 1
        else:
            summary['red'] += 1

        summary['traps'] += int(status['traphits'])
    else:
        data = datetime.strptime(status['activity_end'], '%m/%d/%Y %I:%M %p')
        summary['date'] = data.strftime('%m/%d/%Y')

    return summary


def search_ip_status(ip, response):
    """Porcura pelos status de um IP especifico nos dados de uso de IP (sndslib.get_data).

    >>> r = sndslib.get_data('mykey')
    >>> sndslib.search_ip_status('3.3.3.3', r)
    {'activity_end': '12/31/2019 7:00 PM',
    'activity_start': '12/31/2019 10:00 AM',
    'comments': '',
    'complaint_rate': '< 0.1%',
    'data_commands': '1894',
    'filter_result': 'GREEN',
    'ip_address': '3.3.3.3',
    'message_recipients': '1894',
    'rcpt_commands': '1895',
    'sample_helo': '',
    'sample_mailfrom': '',
    'trap_message_end': '',
    'trap_message_start': '',
    'traphits': '0'}
    """

    for line in response:
        if re.search(ip, line):
            line = line.split(',')
            break
    else:
        return False

    ip_data = format_ip_data(line)

    return ip_data


def format_ip_data(ip_status):
    ip_data = {
        'ip_address': ip_status[0],
        'activity_start': ip_status[1],
        'activity_end': ip_status[2],
        'rcpt_commands': ip_status[3],
        'data_commands': ip_status[4],
        'message_recipients': ip_status[5],
        'filter_result': ip_status[6],
        'complaint_rate': ip_status[7],
        'trap_message_start': ip_status[8],
        'trap_message_end': ip_status[9],
        'traphits': ip_status[10],
        'sample_helo': ip_status[11],
        'sample_mailfrom': ip_status[11],
        'comments': ip_status[12],
    }

    return ip_data


def list_blocked_ips(response):
    """Calcula a lista de IPs bloqueados com base na lista de ranges bloqueados (sndslib.get_ip_status).

    >>> sndslib.get_ip_status('mykey')
    ['1.1.1.1,1.1.1.3,Yes,Blocked due to user complaints or other evidence of spamming']
    >>> sndslib.list_blocked_ips(r)
    [1.1.1.1, 1.1.1.2, 1.1.1.3]
    """

    # Lista que receberá o total de IPs bloqueados
    lista = []

    rangestart = []
    rangeend = []

    # Calcula a diferença entre IP de inicio fim do range bloqueado
    for x in range(len(response) - 1):
        rangestart.append(response[x].split(',')[0])
        rangeend.append(response[x].split(',')[1])

        # Adiciona primeiro IP a lista de bloqueado a lista
        lista.append(rangestart[x])

        # Quebra os octetos do IP para calcular a diferenca entre IP inicial e final
        inicial = rangestart[x].split('.')
        final = rangeend[x].split('.')

        # Calcula o próximo IP bloqueado se existir mais de um no range (inicial != final)
        while inicial != final:
            if int(inicial[3]) < 255:
                inicial[3] = str(int(inicial[3]) + 1)
            elif int(inicial[2]) < 255:
                inicial[2] = str(int(inicial[2]) + 1)
                inicial[3] = '0'
            elif int(inicial[1]) < 255:
                inicial[1] = str(int(inicial[1]) + 1)
                inicial[2] = inicial[3] = '0'
            elif int(inicial[1]) < 255:
                inicial[0] = str(int(inicial[0]) + 1)
                inicial[1] = inicial[2] = inicial[3] = '0'

            # Adiciona IP atualizado a lista
            lista.append('.'.join(inicial))

    return lista


def list_blocked_ips_rdns(ips: list) -> list:
    """Busca o host de uma lista de endereços IP (sndslib.list_blocked_ips).

    >>> sndslib.list_blocked_ips_rdns(['1.1.1.1', '1.1.1.2'])
    [{'ip': '1.1.1.1', 'rdns': 'foo.bar.exemple.com'},
     {'ip': '1.1.1.2', 'rdns': 'foo2.bar.exemple.com'}]
    """

    data = []

    if not isinstance(ips, list):
        # Caso seja passado apenas um IP
        try:
            ips = str(ips)
            rdns = socket.gethostbyaddr(ips)[0]
            data.append({'ip': ips, 'rdns': rdns})
        except socket.error as e:
            # 'socket.gethostbyaddr' levanta exceção caso o IP não tenha rdns
            data.append({'ip': ips, 'rdns': str(e)})

        return data
    else:
        # Caso seja passada uma lista de IPs
        for ip in ips:
            try:
                rdns = socket.gethostbyaddr(ip)[0]
                data.append({'ip': str(ip), 'rdns': rdns})
            except socket.error as e:
                data.append({'ip': str(ip), 'rdns': str(e)})

        return data



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sndslib']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['snds = sndslib.cli:main']}

setup_kwargs = {
    'name': 'sndslib',
    'version': '0.1.1',
    'description': 'Process and verify data from SNDS easily',
    'long_description': "# SNDS LIB\n\n[![Build Status](https://travis-ci.org/undersfx/sndslib.svg?branch=master)](https://travis-ci.org/undersfx/sndslib) [![codecov](https://codecov.io/gh/undersfx/sndslib/branch/master/graph/badge.svg)](https://codecov.io/gh/undersfx/sndslib) [![Python 3](https://pyup.io/repos/github/undersfx/sndslib/python-3-shield.svg)](https://pyup.io/repos/github/undersfx/sndslib/) [![Updates](https://pyup.io/repos/github/undersfx/sndslib/shield.svg)](https://pyup.io/repos/github/undersfx/sndslib/)\n\nProcess and verify data from Microsoft's Smart Network Data Service (SNDS) API easily.\n\nSNDSLIB is a wrapper around SNDS Automated Data Access API to facilitate fast data process and analysis.\n\n\n## What is SNDS?\n\nSmart Network Data Service (SNDS) is a platform to monitor data from IPs that send emails to Microsoft's servers. If you send more than 100 messages per day from your IPs, your can get valuable information about IP reputation, possible blocks, spam complaints and spamtraps hits.\n\n\n## Talk is cheap. Show me the code!\n\nSimple example of library use:\n\n```python\n    >>> from sndslib import sndslib\n\n    >>> r = sndslib.get_ip_status('mykey')\n    >>> blocked_ips = sndslib.list_blocked_ips(r)\n    [1.1.1.1, 2.2.2.2, 3.3.3.3]\n\n    >>> r = sndslib.get_data('mykey')\n    >>> sndslib.summarize(r)\n    {'red': 272, 'green': 710, 'yellow': 852, 'traps': 1298, 'ips': 1834, 'date': '12/31/2019'}\n\n    >>> sndslib.search_ip_status('3.3.3.3', r)\n    {'activity_end': '12/31/2019 7:00 PM',\n    'activity_start': '12/31/2019 10:00 AM',\n    'comments': '',\n    'complaint_rate': '< 0.1%',\n    'data_commands': '1894',\n    'filter_result': 'GREEN',\n    'ip_address': '3.3.3.3',\n    'message_recipients': '1894',\n    'rcpt_commands': '1895',\n    'sample_helo': '',\n    'sample_mailfrom': '',\n    'trap_message_end': '',\n    'trap_message_start': '',\n    'traphits': '0'}\n```\n\n\n## CLI\n\nThis library contains a CLI to facilitate fast operations in the terminal.\n\nSome examples:\n\nSummary of all IPs status\n```bash\nsnds -k 'your-key-here' -s\n```\n\nIndividual report of a IP\n```bash\nsnds -k 'your-key-here' -ip '1.1.1.1'\n```\n\nList all IPs blocked\n```bash\nsnds -k 'your-key-here' -l\n```\n\nList all IPs blocked with rDNS\n```bash\nsnds -k 'your-key-here' -r\n```\n\n\nMore information in the [SNDS](https://sendersupport.olc.protection.outlook.com/snds/FAQ.aspx?wa=wsignin1.0) and [SNDS Automated Data Access](https://sendersupport.olc.protection.outlook.com/snds/auto.aspx) pages.\n",
    'author': 'undersfx',
    'author_email': 'undersoft.corp@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/undersfx/sndslib',
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python3
# snds cli by @undersfx

from __future__ import absolute_import
from sndslib import sndslib
from argparse import ArgumentParser
from .__version__ import __version__


def print_list_blocked_ips(blocked_ips):
    print('\n'.join(blocked_ips))


def print_list_blocked_ips_rdns(blocked_ips_rdns):
    for ip in blocked_ips_rdns:
        print(ip['ip'] + ';' + ip['rdns'])


def print_ip_status(summary, blocked_ips):
    message = (
        f"Date: {summary['date']:>9} \n"
        f"IPs: {summary['ips']:>10} \n"
        f"Green: {summary['green']:>8} \n"
        f"Yellow: {summary['yellow']:>7} \n"
        f"Red: {summary['red']:>10} \n"
        f"Trap Hits: {summary['traps']:>4} \n"
        f"Blocked: {len(blocked_ips):>6}"
    )

    print(message)


def print_ip_data(ipdata):
    message = (
        f"Activity: {ipdata['activity_start']} until {ipdata['activity_end']} \n"
        f"IP: {ipdata['ip_address']:>15} \n"
        f"Messages: {ipdata['message_recipients']:>9} \n"
        f"Filter: {ipdata['filter_result']:>11} \n"
        f"Complaint: {ipdata['complaint_rate']:>8} \n"
        f"Trap Hits: {ipdata['traphits']:>8} \n"
    )

    print(message)


# Argument's instructions
parser = ArgumentParser(prog='snds',
                        description='Searches and formats the SNDS \
                            dashboard data',
                        epilog='You can also use a configuration file using @ \
                            (e.g. "snds -s @parameters.txt")',
                        fromfile_prefix_chars='@')

parser.add_argument('-V', '--version', action='version', version=f'sndslib {__version__}',
                    help='returns the version of sndslib')

parser.add_argument('-k', action='store', dest='key',
                    help='snds access key automated data access',
                    required=True)

parser.add_argument('-d', action='store', dest='data',
                    help='returns the general status on informed date (format=MMDDYY)')

group1 = parser.add_mutually_exclusive_group()

group1.add_argument('-s', action='store_true',
                    help='returns the general status of the most recent data')

group1.add_argument('-ip', action='store',
                    help='returns the complete status of informed IP')

group1.add_argument('-l', action='store_true',
                    help='returns the blocked IPs list')

group1.add_argument('-r', action='store_true',
                    help='returns the blocked IPs list with reverses')


# Parse and execution of the arguments
def main():
    args = parser.parse_args()

    # Conection with SNDS
    try:
        if args.s or args.ip:
            if args.data:
                rdata = sndslib.get_data(args.key, args.data)
            else:
                rdata = sndslib.get_data(args.key)

        if not args.ip:
            rstatus = sndslib.get_ip_status(args.key)
            blocked_ips = sndslib.list_blocked_ips(rstatus)
    except AssertionError as e:
        print(e)
        return

    # Arguments execution chain
    if args.r:
        rdns = sndslib.list_blocked_ips_rdns(blocked_ips)
        print_list_blocked_ips_rdns(rdns)
    elif args.l:
        print_list_blocked_ips(blocked_ips)

    if args.s:
        summary = sndslib.summarize(rdata)
        print_ip_status(summary, blocked_ips)
    elif args.ip:
        ipdata = sndslib.search_ip_status(args.ip, rdata)
        if ipdata:
            print_ip_data(ipdata)
        else:
            print('IP not found')


if __name__ == '__main__':
    main()
